import re
from playwright.sync_api import expect
from robotlibcore import keyword
from .ui_context import UIContext
from .utils import Robot


class DropdownHandler(UIContext):

    @keyword('dropdown should be enabled')
    def dropdown_should_be_enabled(self, locator):
        expect(self.get_element(locator)).to_be_enabled()

    @keyword('dropdown should be disabled')
    def dropdown_should_be_disabled(self, locator):
        expect(self.get_element(locator)).to_be_disabled()

    @keyword('dropdown should be required')
    def dropdown_should_be_required(self, locator):
        element = self.get_element(locator).locator("xpath=./preceding-sibling::*")
        expect(element).to_have_class(re.compile(r"required"))

    @keyword('dropdown should not be required')
    def dropdown_should_not_be_required(self, locator):
        element = self.get_element(locator).locator("xpath=./preceding-sibling::*")
        expect(element).not_to_have_class(re.compile(r"required"))

    @keyword('select value')
    def select_value(self, locator, selected_value):
        self.get_element(locator).select_option(label=selected_value)

    @keyword('select nested value', tags=["specific"])
    def select_nested_value(self, locator, selected_value, wait_for_value=True, unchecked=False):
        element = self.get_element(locator)
        target_label = element.locator(
            f'xpath=/following-sibling::div//label[text()="{selected_value}"]').locator('nth=0')
        element.click()
        if wait_for_value:
            expect(target_label).to_be_visible()
        checkbox = target_label.locator('xpath=/preceding-sibling::input')
        checkbox.check() if not unchecked else checkbox.uncheck()
        element.click(force=True)

    @keyword('nested dropdown should contain', tags=["specific"])
    def nested_dropdown_should_contain(self, locator, *values):
        element = self.get_element(locator)
        element.click(force=True)
        for value in values:
            expect(element.locator(
                f'xpath=/following-sibling::div//label[text()="{value}"]').locator('nth=0')).to_be_visible()
        element.click(force=True)

    @keyword('nested item should be ticked', tags=["specific"])
    def nested_item_should_be_ticked(self, locator, *values):
        element = self.get_element(locator)
        element.click(force=True)
        for value in values:
            expect(element.locator(
                f'xpath=./following-sibling::div//div[@title="{value}"]/input')).to_be_checked()
        element.click(force=True)

    @keyword('nested item should not be ticked', tags=["specific"])
    def nested_item_should_not_be_ticked(self, locator, *values):
        element = self.get_element(locator)
        element.click(force=True)
        for value in values:
            expect(element.locator(
                f'xpath=./following-sibling::div//div[@title="{value}"]/input')).not_to_be_checked()
        element.click(force=True)

    @keyword('nested item should be enabled', tags=["specific"])
    def nested_item_should_be_enabled(self, locator, *values):
        element = self.get_element(locator)
        element.click(force=True)
        for value in values:
            expect(element.locator(
                f'xpath=./following-sibling::div//div[@title="{value}"]/input')).to_be_enabled()
        element.click(force=True)

    @keyword('nested item should be disabled', tags=["specific"])
    def nested_item_should_be_disabled(self, locator, *values):
        element = self.get_element(locator)
        element.click(force=True)
        for value in values:
            expect(element.locator(
                f'xpath=./following-sibling::div//div[@title="{value}"]/input')).to_be_disabled()
        element.click(force=True)

    @keyword('dropdown itemlist should be')
    def dropdown_itemlist_should_be(self, locator, item_list):
        """
        [Documentation]    Verify combobox at ${locator} having correct item list as ${item_list}
        ${item_list} needs to be separated with ';' for each value
        E.g: Robot;Agent;Sale;Customer
        :param locator: element's locator
        :param item_list: Robot;Agent;Sale;Customer
        :return: assert the expected result
        """
        assert self.get_list_values(locator) == item_list.split(
            ";"), f'{self.get_list_values(locator)} is not equal to {item_list.split(";")}'

    @keyword('list item should be')
    def list_item_should_be(self, locator, *item_list):
        """
        [Documentation]    Verify combobox at ${locator} having correct item list as ${item_list}
        ${item_list} needs to be separated with ';' for each value
        E.g: Robot;Agent;Sale;Customer
        :param locator: element's locator - string
        :param item_list: list of string
        :return: assert the expected result
        """
        assert self.get_list_values(locator) == list(item_list), f'{self.get_list_values(locator)} is not equal to ' \
                                                                 f'{item_list}'

    @keyword('dropdown itemlist should contain')
    def dropdown_itemlist_should_contain(self, locator, item_list):
        Robot().should_contain(self.get_list_values(locator), item_list.split(";"))

    @keyword('dropdown itemlist should not contain')
    def dropdown_itemlist_should_not_contain(self, locator, item_list):
        Robot().should_not_contain(self.get_list_values(locator), item_list.split(";"))

    @keyword('dropdown current value should be')
    def dropdown_current_value_should_be(self, locator, expected_value):
        element = self.get_element(locator).locator('xpath=./option[@selected="selected"]')
        expect(element).to_have_text(expected_value)

    @keyword('dropdown current value should not be')
    def dropdown_current_value_should_not_be(self, locator, expected_value):
        element = self.get_element(locator).locator('xpath=./option[@selected="selected"]')
        expect(element).not_to_have_value(expected_value)

    @keyword('dropdown current value should contain')
    def dropdown_current_value_should_contain(self, locator, expected_value):
        element = self.get_element(locator).locator('xpath=./option[@selected="selected"]')
        expect(element).to_contain_text(expected_value)

    @keyword('dropdown current value should not contain')
    def dropdown_current_value_should_not_contain(self, locator, expected_value):
        element = self.get_element(locator).locator('xpath=./option[@selected="selected"]')
        expect(element).not_to_contain_text(expected_value)

    @keyword('get current selected value')
    def get_current_selected_value(self, locator):
        element = self.get_element(locator).locator('xpath=./option[@selected="selected"]')
        return element.input_value()

    @keyword('get list values')
    def get_list_values(self, locator):
        return self.get_element(locator).locator("xpath=./option").all_text_contents()

    @keyword('dropdown should be correct')
    def dropdown_should_be_correct(self, locator, item_list=None, state='enabled', default=None, mandatory=None):
        element = self.get_element(locator)
        # Verify state
        if state == 'enabled':
            self.dropdown_should_be_enabled(element)
        elif state == 'disabled':
            self.dropdown_should_be_disabled(element)
        # Verify default value
        if default is not None:
            self.dropdown_current_value_should_be(element, default)
        # Verify item list
        if item_list is not None:
            self.dropdown_itemlist_should_be(element, item_list)
        # Verify mandatory:
        if mandatory == 'unrequired':
            self.dropdown_should_not_be_required(locator)
        elif mandatory == 'required':
            self.dropdown_should_be_required(locator)

    @keyword('nested dropdown should be correct')
    def nested_dropdown_should_be_correct(self, locator, item_list=None, state='enabled', default=None):
        element = self.get_element(locator)
        # Verify state
        if state == 'enabled':
            expect(element).to_be_enabled()
        elif state == 'disabled':
            assert "disabled" in element.get_attribute("class"), f"{locator} is not disabled"
        # Verify default value
        if default is not None:
            placeholder = element.locator('xpath=./div/p')
            assert default in placeholder.text_content(), f"{locator} is having default value: {placeholder.text}"
        # Verify item list
        if item_list is not None:
            self.nested_dropdown_should_contain(locator, *item_list.split(";"))

    @keyword('get selected item index')
    def get_selected_item_index(self, locator):
        item_list = self.get_list_values(locator)
        return item_list.index(self.get_element(locator).locator("xpath=./option").text_content())

    @keyword('get item list length')
    def get_item_list_length(self, locator):
        return len(self.get_list_values(locator))



from robotlibcore import keyword


class Misc:

    @keyword('setup custom locator', tags=["deprecated"])
    def setup_custom_locator(self):
        pass

    @keyword('Set Library Search Order', tags=["deprecated"])
    def set_library_search_order(self):
        pass

    @keyword('wait for page to be ready', tags=["deprecated"])
    def wait_for_page_to_be_ready(self):
        pass



from playwright.sync_api import expect
from robotlibcore import keyword
from .ui_context import UIContext
from .utils import Robot
from .base_context import TIMEOUT, SMALL_TIMEOUT


class PopupHandler(UIContext):

    @keyword('popup should be shown', tags=["specific"])
    def popup_should_be_shown(self, title, timeout=TIMEOUT):
        if '"' in title:
            expect(self.get_element(f"//*[@class='modal-title'][text()='{title}']|"
                                    f"//*[@class='modal-title']/*[text()='{title}']")).to_be_visible(timeout=timeout)
        else:
            expect(self.get_element(f'//*[@class="modal-title"][text()="{title}"]|'
                                    f'//*[@class="modal-title"]/*[text()="{title}"]')).to_be_visible(timeout=timeout)

    @keyword('popup should not be shown', tags=["specific"])
    def popup_should_not_be_shown(self, title, timeout=TIMEOUT):
        if '"' in title:
            expect(self.get_element(f"//*[@class='modal-title'][text()='{title}']|"
                                    f"//*[@class='modal-title']/*[text()='{title}']")).to_be_hidden(timeout=timeout)
        else:
            expect(self.get_element(f'//*[@class="modal-title"][text()="{title}"]|'
                                    f'//*[@class="modal-title"]/*[text()="{title}"]')).to_be_hidden(timeout=timeout)

    @keyword('popup title should have', tags=["specific"])
    def popup_title_should_have(self, title, timeout=TIMEOUT):
        if '"' in title:
            expect(self.get_element(f"//*[@class='modal-title'][contains(.,'{title}')]")).to_be_visible(timeout=timeout)
        else:
            expect(self.get_element(f'//*[@class="modal-title"][contains(.,"{title}")]')).to_be_visible(timeout=timeout)

    @keyword('popup should be closed', tags=["specific"])
    def popup_should_be_closed(self, title=None, timeout=TIMEOUT):
        if not title:
            expect(self.get_element(f'//*[@class="modal-dialog"]')).to_be_hidden(timeout=timeout)
        else:
            if '"' in title:
                expect(self.get_element(f"//*[@class='modal-title'][text()='{title}']")).to_be_hidden(timeout=timeout)
            else:
                expect(self.get_element(f'//*[@class="modal-dialog"]')).to_be_hidden(timeout=timeout)

    @keyword('popup should have', tags=["specific"])
    def popup_should_have(self, *texts, timeout=TIMEOUT):
        for text in texts:
            element = self.get_element('//*[@class="modal-content"]').locator(f"text={text}")
            expect(element).to_be_visible(timeout=timeout)

    @keyword('popup should not have', tags=["specific"])
    def popup_should_not_have(self, expected_text, timeout=SMALL_TIMEOUT):
        element = self.get_element('//*[@class="modal-content"]').locator(f"text={expected_text}")
        expect(element).to_be_hidden(timeout=timeout)

    @keyword('popup should have element', tags=["specific"])
    def popup_should_have_element(self, xpath):
        element = self.get_element('//*[@class="modal-content"]').locator(xpath)
        expect(element).to_be_visible(timeout=SMALL_TIMEOUT)

    @keyword('popup should not have element', tags=["specific"])
    def popup_should_not_have_element(self, xpath):
        element = self.get_element('//*[@class="modal-content"]').locator(xpath)
        expect(element).to_be_hidden(timeout=SMALL_TIMEOUT)

    @keyword('wait for popup to open', tags=["specific"])
    def wait_for_popup_to_open(self, popup_title, timeout=SMALL_TIMEOUT):
        expect(self.get_element(f'//*[@class="modal-title" and text()="{popup_title}"]')).to_be_visible(timeout=timeout)

    @keyword('wait for popup to close', tags=["specific"])
    def wait_for_popup_to_close(self, popup_title, timeout=SMALL_TIMEOUT):
        expect(self.get_element(f'//*[@class="modal-title" and text()="{popup_title}"]')).to_be_hidden(timeout=timeout)

    @keyword('close the popup', tags=["specific"])
    def close_the_popup(self):
        self.get_element('//a[text()="×"]').click()

    @keyword('close any popup', tags=["specific"])
    def close_any_popup(self):
        Robot().sleep(3)
        close_btn = '//a[text()="×"]'
        count = self.get_element(close_btn).count()
        if count > 0:
            self.get_element(close_btn).locator("nth=0").click()



from datetime import datetime
from playwright.sync_api import expect
from robotlibcore import keyword
from .ui_context import UIContext
from .utils import Robot
from .base_context import SMALL_TIMEOUT


class DatePickerHandler(UIContext):

    BASIC_DATE_FORMAT = "%Y-%m-%d"
    ALTERNATIVE_DATE_FORMAT = "%d %b, %Y"

    @keyword('input datetime')
    def input_datetime(self, locator, value):
        element = self.get_element(locator)
        element.evaluate("node => node.removeAttribute('readonly')")
        element.fill(value, force=True)
        element.locator("xpath=./following-sibling::span/button").click()
        Robot().sleep(1)
        return value

    @keyword('datepicker should be correct')
    def datepicker_should_be_correct(self, locator, state='enabled', default=None, mandatory=None):
        element = self.get_element(locator)
        # Verify state
        if state == 'enabled':
            expect(element).to_be_enabled()
        elif state == 'disabled':
            expect(element).to_be_disabled()
        # Verify default value
        if default is not None:
            expect(element).to_have_value(default)
        # Verify mandatory
        if mandatory == 'unrequired':
            self.element_should_not_be_marked_as_required(locator)
        elif mandatory == 'required':
            self.element_should_be_marked_as_required(locator)

    @keyword('actual date should be')
    def actual_date_should_be(self, locator, expected_date, input_format=BASIC_DATE_FORMAT,
                              displayed_format=ALTERNATIVE_DATE_FORMAT, timeout=SMALL_TIMEOUT):
        actual_date = None
        for sec in range(int(timeout/1000)):
            actual_date = self.get_actual_text(locator)
            if datetime.strptime(expected_date, input_format) == datetime.strptime(actual_date, displayed_format):
                break
            Robot().sleep(1)
        if datetime.strptime(expected_date, input_format) != datetime.strptime(actual_date, displayed_format):
            raise AssertionError(f"Actual date: '{actual_date}' is different with expected date: '{expected_date}'")

    @keyword('actual date should not be')
    def actual_date_should_not_be(self, locator, expected_date, input_format=BASIC_DATE_FORMAT,
                                  displayed_format=ALTERNATIVE_DATE_FORMAT, timeout=SMALL_TIMEOUT):
        actual_date = None
        for sec in range(int(timeout/1000)):
            actual_date = self.get_actual_text(locator)
            if datetime.strptime(expected_date, input_format) != datetime.strptime(actual_date, displayed_format):
                break
            Robot().sleep(1)
        if datetime.strptime(expected_date, input_format) == datetime.strptime(actual_date, displayed_format):
            raise AssertionError(f"Actual date: '{actual_date}' is similar with expected text: '{expected_date}'")

    @keyword("element is datepicker", tags=["deprecated", "specific"])
    def element_is_datepicker(self, locator):
        return True if self.get_element(locator).get_attribute(locator, "uib-datepicker-popup") is not None else False



from robotlibcore import DynamicCore
from .api_context import APIContext
from .button_handler import ButtonHandler
from .checkbox_handler import CheckboxHandler
from .datepicker_handler import DatePickerHandler
from .dropdown_handler import DropdownHandler
from .iframe_handler import IframeHandler
from .page_handler import PageHandler
from .popup_handler import PopupHandler
from .table_handler import TableHandler
from .textbox_handler import TextboxHandler
from .ui_context import UIContext
from .misc import Misc


class PlayerLibrary(DynamicCore):

    def __init__(self):
        libraries = [APIContext(), UIContext(), ButtonHandler(), CheckboxHandler(), DatePickerHandler(),
                     DropdownHandler(), IframeHandler(), PageHandler(), PopupHandler(), TableHandler(),
                     TextboxHandler(), Misc()]
        DynamicCore.__init__(self, libraries)



from playwright.sync_api import expect
from robotlibcore import keyword
from .ui_context import UIContext
from .base_context import SMALL_TIMEOUT


class CheckboxHandler(UIContext):

    @keyword('checkbox should be enabled')
    def checkbox_should_be_enabled(self, locator):
        element = self.get_element(locator)
        expect(element).to_be_enabled(timeout=SMALL_TIMEOUT)

    @keyword('checkbox should be disabled')
    def checkbox_should_be_disabled(self, locator):
        element = self.get_element(locator)
        expect(element).to_be_disabled(timeout=SMALL_TIMEOUT)

    @keyword('tick checkbox')
    def tick_checkbox(self, locator):
        element = self.get_element(locator)
        element.check()

    @keyword('untick checkbox')
    def untick_checkbox(self, locator):
        element = self.get_element(locator)
        element.uncheck()

    @keyword('checkbox should be checked')
    def checkbox_should_be_checked(self, locator):
        expect(self.get_element(locator)).to_be_checked()

    @keyword('checkbox should not be checked')
    def checkbox_should_not_be_checked(self, locator):
        expect(self.get_element(locator)).not_to_be_checked()

    @keyword('get current checkbox checking status')
    def get_current_checkbox_checking_status(self, locator):
        return self.get_element(locator).is_checked()

    @keyword('checkbox should be correct')
    def checkbox_should_be_correct(self, locator, state='enabled', status='unchecked'):
        element = self.get_element(locator)
        # Verify state
        if state == 'enabled':
            self.checkbox_should_be_enabled(element)
        elif state == 'disabled':
            self.checkbox_should_be_disabled(element)
        # Verify status:
        if status == 'unchecked':
            self.checkbox_should_not_be_checked(element)
        elif status == 'checked':
            self.checkbox_should_be_checked(element)

    @keyword('select a radio option')
    def select_a_radio_option(self, locator):
        self.get_element(locator).check(force=True)

    @keyword('radio button should be disabled')
    def radio_button_should_be_disabled(self, locator):
        expect(self.get_element(locator)).to_be_disabled()

    @keyword('radio button should be enabled')
    def radio_button_should_be_enabled(self, locator):
        expect(self.get_element(locator)).to_be_enabled()

    @keyword('radio button should be checked')
    def radio_button_should_be_checked(self, locator):
        expect(self.get_element(locator)).to_be_checked()

    @keyword('radio button should not be checked')
    def radio_button_should_not_be_checked(self, locator):
        expect(self.get_element(locator)).not_to_be_checked()

    @keyword('get current radio button checking status')
    def get_current_radio_button_checking_status(self, locator):
        return self.get_element(locator).is_checked()

    @keyword('radio should be correct')
    def radio_should_be_correct(self, locator, state='enabled', status='unchecked'):
        element = self.get_element(locator)
        if state == 'enabled':
            self.checkbox_should_be_enabled(element)
        elif state == 'disabled':
            self.checkbox_should_be_disabled(element)
        if status == 'unchecked':
            self.radio_button_should_not_be_checked(element)
        elif status == 'checked':
            self.radio_button_should_be_checked(element)



import pathlib

from playwright.sync_api import expect
from robotlibcore import keyword
from .utils import Robot, PROJECT_DIR, NOW_TIMESTAMP
from .ui_context import UIContext
from .base_context import SMALL_TIMEOUT, TIMEOUT
from .custom_locator import *


class PageHandler(UIContext):
    def __init__(self):
        super().__init__()

    @keyword('page title should be')
    def page_title_should_be(self, title):
        expect(self.get_page()).to_have_title(title)

    @keyword('get all data of similar html elements')
    def get_all_data_of_similar_html_elements(self, locator):
        elements = self.get_elements(locator)
        return [element.text_content() for element in elements]

    @keyword('text should be visible')
    def text_should_be_visible(self, *texts, timeout=SMALL_TIMEOUT):
        for text in texts:
            element = self.get_element(f'//body//*[not(self::script)][contains(.,"{text}")]').first
            expect(element).to_be_visible(timeout=timeout)

    @keyword('text should not be visible')
    def text_should_not_be_visible(self, *texts, timeout=SMALL_TIMEOUT):
        for text in texts:
            element = self.get_element(f'//body//*[not(self::script)][contains(text(),"{text}")]')
            expect(element).to_be_hidden(timeout=timeout)

    @keyword('texts should be visible')
    def texts_should_be_visible(self, texts: list, timeout=TIMEOUT, deep_scan=False):
        text_node = "text()" if not deep_scan else "."
        for text in texts:
            element = self.get_element(f'//body//*[not(self::script)][contains({text_node},"{text}")]')
            expect(element).to_be_visible(timeout=timeout)

    @keyword('texts should not be visible')
    def texts_should_not_be_visible(self, texts: list, timeout=TIMEOUT, deep_scan=False):
        text_node = "text()" if not deep_scan else "."
        for text in texts:
            element = self.get_element(f'//body//*[not(self::script)][contains({text_node},"{text}")]')
            expect(element).to_be_hidden(timeout=timeout)

    @keyword('Page should have')
    def page_should_have(self, *items, timeout=TIMEOUT):
        for item in items:
            if item.startswith(BUILT_IN_PREFIX + CUSTOM_PREFIX + XPATH_PREFIX):
                self.page_should_have_element(item, timeout)
            else:
                self.text_should_be_visible(item)

    @keyword('page should not have')
    def page_should_not_have(self, *items, timeout=TIMEOUT):
        for item in items:
            if item.startswith(BUILT_IN_PREFIX + CUSTOM_PREFIX + XPATH_PREFIX):
                self.page_should_not_have_element(item, timeout)
            else:
                self.text_should_not_be_visible(item)

    @keyword('page should be blank')
    def page_should_be_blank(self):
        expect(self.get_page()).to_have_url("about:blank")

    @keyword('page should have element')
    def page_should_have_element(self, locator, timeout=TIMEOUT):
        element = self.get_element(locator)
        expect(element).to_be_visible(timeout=timeout)
        return element

    @keyword('page should not have element')
    def page_should_not_have_element(self, locator, timeout=SMALL_TIMEOUT, recheck_timeout=2):
        element = self.get_element(locator)
        expect(element).to_be_hidden(timeout=timeout)
        Robot().sleep(recheck_timeout)
        expect(element).to_be_hidden(timeout=timeout)

    @keyword('page should be redirected to')
    def page_should_be_redirected_to(self, url):
        expect(self.get_page()).to_have_url(url)

    @keyword('alert should be shown')
    def alert_should_be_shown(self, content, locator):
        """
        Verifies that an alert is present and, by default, accepts it.
         > ACCEPT: Accept the alert i.e. press Ok. Default.
         > DISMISS: Dismiss the alert i.e. press Cancel.
         > LEAVE: Leave the alert open.
        :param locator:
        :param content:
        :return:
        """
        with self.get_page().expect_event("dialog") as new_dialog_info:
            self.get_element(locator).click()
        dialog = new_dialog_info.value
        assert dialog.message == content
        dialog.dismiss()

    @keyword('capture screenshot')
    def capture_screenshot(self):
        pathlib.Path(f'{PROJECT_DIR}/results/images').mkdir(parents=True, exist_ok=True)
        path = pathlib.Path(f'{PROJECT_DIR}/results/images/custom-screenshot-{NOW_TIMESTAMP}.png').resolve()
        print(path)
        self.get_page().screenshot(path=path, full_page=True)
        image = f"""
        <html>
            <head>
                <style>
                img.one {{
                  height: 75%;
                  width: 75%;
                }}
                </style>
            </head>
            <body>
                <img class="one" src="{path}">
            </body>
        </html>   
        """
        Robot().log(message=image, html=True)

    @keyword('get page source')
    def get_page_source(self):
        return self.get_page().text_content()

    @keyword('reload whole page')
    def reload_whole_page(self):
        self.get_page().reload()

    @keyword('html title should be')
    def html_title_should_be(self, title):
        expect(self.get_page()).to_have_title(title)

    @keyword('go back to previous page')
    def go_back_to_previous_page(self):
        self.get_page().go_back()

    @keyword("text having correct amount value")
    def text_having_correct_amount_value(self, text, amount):
        """

        :param text: Something like "I have the payout of {abc} will be refunded tomorrow". Should include the
        curly bracket here
        :param amount: The actual amount which will be replaced into {abc}
        :return: Verify if the text having correct expected amount or not
        """
        text = re.sub(r'(?<=\{).*(?=\})', amount, text)
        text = text.replace("{", "").replace("}", "")
        self.page_should_have(text)

    @keyword('upload file', tags=["specific"])
    def upload_file(self, file_path, locator='//input[@type="file"]'):
        """
        Handle the upload file function. Locator must point to the element having 'type=file' attribute
        :param locator:
        :param file_path:
        :return: None
        """
        file_path = file_path.replace('PROJECT_DIR', Robot().get_variable_value('${EXECDIR}'))
        self.get_element(locator).set_input_files(file_path)

    @keyword('wait for text', tags=['deprecated'])
    def wait_for_text(self, text):
        self.text_should_be_visible(text)

    @keyword('wait for text to hide', tags=['deprecated'])
    def wait_for_text_to_hide(self, text):
        self.text_should_not_be_visible(text)

    @keyword('scroll to element', tags=["deprecated"])
    def scroll_to_element(self, locator):
        # self.get_element(locator).scroll_into_view_if_needed()
        pass

    @keyword('scroll element into view', tags=["deprecated"])
    def scroll_element_into_view(self, locator):
        # self.get_element(locator).scroll_into_view_if_needed()
        pass

    @keyword('scroll to element with additional alignment')
    def scroll_to_element_with_additional_alignment(self, locator, alignment='true'):
        """
        [Documentation]    True - the top of the element will be aligned to the top of the visible area of the scrollable ancestor
        ...    False - the bottom of the element will be aligned to the bottom of the visible area of the scrollable ancestor
        ...    If omitted, it will scroll to the top of the element
        :param locator:
        :param alignment:
        :return:
        """
        element = self.get_element(locator)
        element.evaluate(f"scrollIntoView({alignment});")

    @keyword('scroll right')
    def scroll_right(self):
        self.get_page().evaluate("window.scrollTo(document.body.scrollWidth,document.body.scrollHeight);")

    @keyword('scroll down')
    def scroll_down(self):
        self.get_page().evaluate("window.scrollTo(0,document.body.scrollHeight);")



from playwright.sync_api import sync_playwright
from robotlibcore import keyword
from .custom_locator import *


def _get_playwright_context_manager():
    if not BaseContext.playwright_context_manager:
        BaseContext.playwright_context_manager = sync_playwright().start()
    return BaseContext.playwright_context_manager


CHROME_OPTIONS = ["--ignore-certificate-errors", "--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                                                 "AppleWebKit/537.36 "
                                                 "(KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36"]

TIMEOUT = 45000
SMALL_TIMEOUT = 15000
BROWSER = "chrome"
HEADLESS_MODE = True


class BaseContext:

    playwright_context_manager = None

    def __init__(self):
        self.player = _get_playwright_context_manager()

    @keyword("close all browsers")
    def close_all_browsers(self):
        self.player.stop()

    def _setup_custom_locators(self):
        self.player.selectors.register('item', QUERY_BY_ITEM)
        self.player.selectors.register('btn', QUERY_BY_BTN)
        self.player.selectors.register('plc', QUERY_BY_PLC)
        self.player.selectors.register('cbx', QUERY_BY_CBX)
        self.player.selectors.register('radio', QUERY_BY_RADIO)
        self.player.selectors.register('link', QUERY_BY_LINK)
        self.player.selectors.register('name', QUERY_BY_NAME)
        self.player.selectors.register('class', QUERY_BY_CLASS)



import json
from json import JSONDecodeError

from playwright.sync_api import APIResponse, expect

from .base_context import BaseContext
from .utils import pretty_logging
from robotlibcore import keyword


class APIContext(BaseContext):

    DEFAULT_CONTENT_TYPE_JSON = "application/json"
    DEFAULT_CONTENT_TYPE_FORM = "application/x-www-form-urlencoded"
    DEFAULT_HEADER_FORM = {"content-type": "application/x-www-form-urlencoded"}
    DEFAULT_HEADER_JSON = {"content-type": "application/json"}

    def __init__(self):
        super().__init__()
        self.api = self.player.request.new_context()

    @keyword("rest post")
    def rest_post(self, url, headers, body, code=200):
        response = self.api.post(
            url,
            headers=json.loads(headers),
            data=body,
        )
        try:
            resp_body = response.json()
        except (JSONDecodeError, TypeError):
            resp_body = str(response.body())
        self.http_request_should_be_successful(response, code)
        pretty_logging(url)
        pretty_logging(headers)
        pretty_logging(body)
        pretty_logging(response.status)
        pretty_logging(resp_body)
        return resp_body

    @keyword("rest patch")
    def rest_patch(self, url, headers, body, code=200):
        response = self.api.patch(
            url,
            headers=json.loads(headers),
            data=body,
        )
        try:
            resp_body = response.json()
        except (JSONDecodeError, TypeError):
            resp_body = str(response.body())
        self.http_request_should_be_successful(response, code)
        pretty_logging(url)
        pretty_logging(headers)
        pretty_logging(body)
        pretty_logging(response.status)
        pretty_logging(resp_body)
        return resp_body

    @keyword("rest put")
    def rest_put(self, url, headers, body, code=200):
        response = self.api.put(
            url,
            headers=json.loads(headers),
            data=body,
        )
        try:
            resp_body = response.json()
        except (JSONDecodeError, TypeError):
            resp_body = str(response.body())
        self.http_request_should_be_successful(response, code)
        pretty_logging(url)
        pretty_logging(headers)
        pretty_logging(body)
        pretty_logging(response.status)
        pretty_logging(resp_body)
        return resp_body

    @keyword("rest delete")
    def rest_delete(self, url, headers, body, code=200):
        response = self.api.delete(
            url,
            headers=json.loads(headers),
            data=body,
        )
        try:
            resp_body = response.json()
        except (JSONDecodeError, TypeError):
            resp_body = str(response.body())
        self.http_request_should_be_successful(response, code)
        pretty_logging(url)
        pretty_logging(headers)
        pretty_logging(body)
        pretty_logging(response.status)
        pretty_logging(resp_body)
        return resp_body

    @keyword("rest get")
    def rest_get(self, url, headers, code=200):
        response = self.api.get(
            url,
            headers=json.loads(headers)
        )
        try:
            resp_body = response.json()
        except (JSONDecodeError, TypeError):
            resp_body = str(response.body())
        self.http_request_should_be_successful(response, code)
        pretty_logging(url)
        pretty_logging(headers)
        pretty_logging(response.status)
        pretty_logging(resp_body)
        return resp_body

    def rest_dispose(self):
        self.api.dispose()

    @keyword("http status code should be", tags=["deprecated"])
    def http_status_code_should_be(self, code):
        pass

    @keyword("http request should be successful")
    def http_request_should_be_successful(self, response: APIResponse, code):
        if str(code).startswith("2"):
            expect(response).to_be_ok()
        else:
            expect(response).not_to_be_ok()

    @keyword("create header")
    def create_header(self, token, content_type=DEFAULT_CONTENT_TYPE_JSON, x_time_travel_date=None, **kwargs):
        """
         Supported content-types:
         > application/json
         > application/x-www-form-urlencoded
         > multipart/form-data
         > undefined
        :param token:
        :param content_type:
        :param x_time_travel_date:
        :param kwargs:
        :return:
        """
        header = kwargs
        if token:
            header["Authorization"] = f"Bearer {token}"
        if content_type:
            header["content-type"] = content_type
        if x_time_travel_date:
            header["x-time-travel-date"] = x_time_travel_date
        return json.dumps(header)
